@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2015 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2016 by Andrew Mustun. All rights reserved.
  * 
  * This file is part of the QCAD project.
  *
@@ -350,6 +350,9 @@ ShapeAlgorithms.getOffsetArcs = function(shape, distance, number, sidePosition)
     return ret;
 };
 
+/**
+ * \return Array of spline shapes representing the parallel curves to the given ellipse shape.
+ */
 ShapeAlgorithms.getOffsetEllipses = function(shape, distance, number, sidePosition) {
     var ret = [];
     var i, n;
@@ -358,6 +361,10 @@ ShapeAlgorithms.getOffsetEllipses = function(shape, distance, number, sidePositi
 
     center = shape.getCenter();
 
+    if (shape.isReversed()) {
+        shape.reverse();
+    }
+
     insides = [];
     if (isVector(sidePosition)) {
         a = center.getAngleTo(sidePosition) - shape.getAngle();
@@ -371,13 +378,11 @@ ShapeAlgorithms.getOffsetEllipses = function(shape, distance, number, sidePositi
             insides.push(false);
         }
         else {
-            if (!shape.isReversed()) {
-                if (sidePosition===RS.LeftHand) {
-                    insides.push(true);
-                }
-                else {
-                    insides.push(false);
-                }
+            if (sidePosition===RS.LeftHand) {
+                insides.push(true);
+            }
+            else {
+                insides.push(false);
             }
         }
     }
@@ -403,13 +408,23 @@ ShapeAlgorithms.getOffsetEllipses = function(shape, distance, number, sidePositi
             }
 
             var endParam = shape.getEndParam();
+            var startParam = shape.getStartParam();
             if (RMath.fuzzyCompare(endParam, 0.0)) {
                 endParam = 2*Math.PI;
             }
 
+            if (endParam<startParam) {
+                endParam += 2*Math.PI;
+            }
+
             var k = d*n;
             var root, v;
-            for (t=shape.getStartParam(); t<endParam+0.1; t+=0.1) {
+            var tMax = endParam+0.1;
+            if (isFullEllipseShape(shape)) {
+                tMax = endParam;
+            }
+
+            for (t=startParam; t<tMax; t+=0.1) {
                 if (t>endParam) {
                     t = endParam;
                 }
@@ -427,6 +442,11 @@ ShapeAlgorithms.getOffsetEllipses = function(shape, distance, number, sidePositi
                     spl.appendVertex(v);
                 }
             }
+
+            if (isFullEllipseShape(shape)) {
+                spl.setPeriodic(true);
+            }
+
             ret.push(spl);
         }
     }
@@ -1141,7 +1161,7 @@ ShapeAlgorithms.createArcFrom3Points = function(point1, point2, point3) {
 
     var ret = RArc.createFrom3Points(point1, point2, point3);
 
-    if (isNull(ret)) {
+    if (!ret.isValid()) {
         ShapeAlgorithms.error = qsTr("No arc possible.");
         return undefined;
     }
