@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2015 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2016 by Andrew Mustun. All rights reserved.
  * 
  * This file is part of the QCAD project.
  *
@@ -342,6 +342,9 @@ EventHandler.prototype.updateSnapInfo = function(painter, snap, restriction) {
     case RSnap.Perpendicular:
         text = qsTr("Perpendicular");
         break;
+    case RSnap.Tangential:
+        text = qsTr("Tangential");
+        break;
     case RSnap.Center:
         text = qsTr("Center");
         break;
