@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2015 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2016 by Andrew Mustun. All rights reserved.
  * 
  * Function stringToDirectDistanceEntry added 2013 by Robert S.
  *
@@ -445,8 +445,13 @@ function isPolylineEntity(obj) {
 /**
  * \return true if the given object is a geometrically closed polyline entity.
  */
-function isClosedPolylineEntity(obj) {
-    return isPolylineEntity(obj) && obj.isGeometricallyClosed();
+function isClosedPolylineEntity(obj, tolerance) {
+    if (isNull(tolerance)) {
+        return isPolylineEntity(obj) && obj.isGeometricallyClosed();
+    }
+    else {
+        return isPolylineEntity(obj) && obj.isGeometricallyClosed(tolerance);
+    }
 }
 
 /**
@@ -485,6 +490,15 @@ function isSplineEntity(obj) {
     return isOfType(obj, RSplineEntity) || isOfType(obj, RSplineEntityPointer);
 }
 
+function isClosedSplineEntity(obj, tolerance) {
+    if (isNull(tolerance)) {
+        return isSplineEntity(obj) && obj.isGeometricallyClosed();
+    }
+    else {
+        return isSplineEntity(obj) && obj.isGeometricallyClosed(tolerance);
+    }
+}
+
 /**
  * Checks if the given object is a text, attribute or attribute definition entity.
  *
@@ -1302,7 +1316,16 @@ Array.alphaNumericalSorter = function(a, b) {
         // nummerical:
         return aVal - bVal;
     }
-}
+};
+
+/**
+ * Sorts this array with string items (case insensitive)
+ */
+Array.prototype.sortCaseInsensitive = function() {
+    Array.prototype.sort.call(this, function (a, b) {
+        return a.toLowerCase().localeCompare(b.toLowerCase());
+    });
+};
 
 /* Finds the intersection of two sorted arrays.
  *
@@ -2054,8 +2077,9 @@ function restoreOverrideCursor() {
 }
 
 function addActionsToWidgets() {
+    var appWin = RMainWindowQt.getMainWindow();
     var actions = RGuiAction.getActions();
-    var widgetTypes = ["Menu", "ToolBar", "Panel"];
+    var widgetTypes = ["Menu", "ToolBar", "MatrixPanel", "Panel"];
     for (var c=0; c<actions.length; ++c) {
         var a = actions[c];
         if (a.icon.isNull() && !a.isIconDisabled()) {
@@ -2063,6 +2087,7 @@ function addActionsToWidgets() {
         }
         var className = a.getScriptClass();
         var wns = a.getWidgetNames();
+        var addedToWidget = false;
 
         for (var k=0; k<wns.length; k++) {
             var wn = wns[k];
@@ -2091,21 +2116,22 @@ function addActionsToWidgets() {
                         key = className + "/VisibleIn" + keyPostfix;
                         visibility = RSettings.getBoolValue(key, visibility);
                         RSettings.setValue(key, visibility, false);
+                        break;
                     }
                 }
             }
-            var w = RMainWindowQt.getMainWindow().findChild(wn);
+            var w = appWin.findChild(wn);
             if (!isNull(w)) {
 
                 // workaround for QTBUG-38256 (action not triggered for letter based shortcuts in sub menus)
-                if (RSettings.isQt(5)) {
+                if (RSettings.getQtVersion()<0x050500 && RSettings.getQtVersion()>=0x050000) {
                     if (isOfType(w.parentWidget(), QMenu)) {
                         new QShortcut(a.shortcut, a.parentWidget(), 0, 0,  Qt.WindowShortcut).activated.connect(a, "trigger");
 
                         // avoid 'Ambiguous shortcut overload' when tool buttons visible:
                         var sc = a.shortcut.toString();
                         if (sc.length>0) {
-                            a.setShortcutText(" (" + sc + ")");
+                            a.setShortcutText(" " + sc);
                         }
                         a.setDefaultShortcuts([]);
                     }
@@ -2113,6 +2139,7 @@ function addActionsToWidgets() {
 
                 if (visibility) {
                     RGuiAction.addToWidget(a, w);
+                    addedToWidget = true;
 
                     if (isFunction(w.widgetForAction)) {
                         // if action was added to tool bar: set object name of tool button:
@@ -2127,9 +2154,12 @@ function addActionsToWidgets() {
                     RGuiAction.removeFromWidget(a, w);
                 }
             }
-            else {
-                qWarning("RGuiAction::init: Cannot add action to widget: ", wn);
-            }
+        }
+
+        if (!addedToWidget) {
+            // always add action to main window to make sure keycodes
+            // are active even if action is invisible:
+            RGuiAction.addToWidget(a, appWin);
         }
     }
 }
@@ -2166,9 +2196,14 @@ function getUrlsFromMimeData(mimeData) {
 function isUrl(urlString) {
     var url = new QUrl(urlString);
     var scheme = url.scheme();
-    return scheme==="file" || scheme==="http" || scheme==="ftp";
+    return scheme==="file" || scheme==="http" || scheme==="https" || scheme==="ftp";
 }
 
+function getDontUseNativeDialog() {
+    // don't use KDE file dialog (workaround for file type filter bug):
+    return RS.getWindowManagerId()==="kde" || RSettings.getBoolValue("SaveAs/UseSystemFileDialog", true)===false;
+};
+
 // fix QPlainTextEdit API for Qt 5:
 if (!isFunction(QPlainTextEdit.prototype.toPlainText)) {
     QPlainTextEdit.prototype.toPlainText = function() {
